@@ -193,7 +193,8 @@ class TfDataset:
         size = get_flag("size", self.size)
 
         for i, bbox in enumerate(bbs.bounding_boxes):
-            if (bbox.coords < 0).sum() > 0:  # skip any coord < 0 bbox after augmentation
+            # skip any coord < 0 or >= size bbox after augmentation
+            if not 0 < bbox.center_x < size or not 0 < bbox.center_y < size:
                 continue
             y_train[i][0:4] = bbox.x1 / size, bbox.y1 / size, bbox.x2 / size, bbox.y2 / size
             # convert label to one hot
