@@ -26,7 +26,7 @@ from tensorflow.keras.callbacks import (
 )
 
 # self-defined packages
-from utils import os_path, image_processing
+from utils import os_path, image_processing, flag_utils
 from cv.yolov3.core.layers import (
     YoloLoss
 )
@@ -34,14 +34,14 @@ from cv.yolov3.core.models import (
     YoloV3, YoloV3Tiny
 )
 from cv.yolov3.core import utils as yolo_utils
-from utils.keras_utils import LogHistory, plot_history
+from utils.keras_utils import LogHistory, PlotHistory
 
 flags.DEFINE_integer("size", default=416, help="size of image")
 flags.DEFINE_integer('epochs', default=2, help='number of epochs')
 flags.DEFINE_integer('batch_size', default=24, help='batch size')
 flags.DEFINE_integer('early_stop', default=5, help='early stopping')
 flags.DEFINE_float('learning_rate', default=1e-3, help='learning rate')
-flags.DEFINE_integer('freeze', default=-1, help='freeze layers before xth layer')
+flags.DEFINE_integer('freeze', default=0, help='freeze layers before xth layer')
 
 flags.DEFINE_boolean('tiny', False, 'yolov3 or yolov3-tiny')
 flags.DEFINE_string('weights', None, 'path to weights file')
@@ -57,11 +57,10 @@ flags.DEFINE_enum('transfer', 'same',
                   'frozen: Transfer and freeze all, '
                   'fine_tune: Transfer all and freeze darknet only, '
                   'same: Use previous trained model')
+flags.DEFINE_list('loss_weight', [1.0, 1.0, 1.0, 1.0], "[xy, wh, obj, class] loss weight")
 flags.DEFINE_integer('weights_num_classes', None, 'specify num class for `weights` file if different, '
                                                   'useful in transfer learning with different number of classes')
 
-logger.debug(f"Flags: {FLAGS.flags_into_string()}")
-
 
 def run_eager_fit(model, train_dataset, val_dataset, optimizer, loss):
     # Eager mode is great for debugging
@@ -111,6 +110,8 @@ def run_eager_fit(model, train_dataset, val_dataset, optimizer, loss):
 
 @logger.catch(reraise=True)
 def train(model, train_dataset, val_dataset):
+
+    flag_utils.log_flag()
     """Configure the model for transfer learning"""
     if FLAGS.transfer == 'none':
         pass  # Nothing to do
@@ -135,18 +136,17 @@ def train(model, train_dataset, val_dataset):
     else:
         # All other transfer require matching classes
         model.load_weights(weight_path=FLAGS.weights)
-        yolo_utils.freeze_all(model, until_layer=FLAGS.freeze)
-
         if FLAGS.transfer == 'fine_tune':
             # freeze darknet and fine tune other layers
             darknet = model.get_layer('yolo_darknet')
             yolo_utils.freeze_all(darknet, until_layer=FLAGS.freeze)
         elif FLAGS.transfer == 'frozen':
             # freeze everything
-            yolo_utils.freeze_all(model)
+            yolo_utils.freeze_all(model, until_layer=FLAGS.freeze)
 
     optimizer = tf.keras.optimizers.Adam(lr=FLAGS.learning_rate)
-    loss = [YoloLoss(model.anchors[mask], classes=model.num_classes) for mask in model.anchor_masks]
+    loss = [YoloLoss(model.anchors[mask], classes=model.num_classes, weights=FLAGS.loss_weight) for mask in
+            model.anchor_masks]
 
     """Start training"""
     if FLAGS.fit == 'eager_tf':
@@ -157,14 +157,16 @@ def train(model, train_dataset, val_dataset):
         checkpoint_path = os_path.join(os_path.dirname(FLAGS.weights), "checkpoints",
                                        'ep{epoch:03d}-loss{loss:.3f}-val_loss{val_loss:.3f}.h5')
         os_path.make_dir(os_path.dirname(checkpoint_path))
+        plt_his = PlotHistory()
         callbacks = [
             ReduceLROnPlateau(verbose=1),
             EarlyStopping(patience=FLAGS.early_stop, verbose=1),
             ModelCheckpoint(checkpoint_path,
                             monitor='val_loss', verbose=0, save_best_only=False,
-                            save_weights_only=False, mode='auto', save_freq='epoch'),
+                            save_weights_only=True, mode='auto', save_freq='epoch'),
             # TensorBoard(log_dir='../logs'),
-            LogHistory()
+            LogHistory(),
+            plt_his
         ]
         try:
             history = model.fit(train_dataset,
@@ -174,8 +176,8 @@ def train(model, train_dataset, val_dataset):
         except Exception as e:
             raise e
         finally:
-            plot_history(history, save_path=os_path.join(os_path.dirname(FLAGS.weights),
-                                                         f"{datetime.datetime.now().strftime('%Y%m%d-%H%M')}.png"))
+            plt_his.plot(save_path=os_path.join(os_path.dirname(FLAGS.weights),
+                                                f"{datetime.datetime.now().strftime('%Y%m%d-%H%M')}.png"))
 
 
 @logger.catch(reraise=True)
@@ -263,7 +265,10 @@ def test(model, test_dataset, voc_set, box_thres=15):
                       suffixes=("", "_true")).reset_index(drop=True)
 
     # box_diff are huge (false positive)
-    false_positive_idx = df_all[df_all["diff"] > thres].index
+    df_all["w"] = df_all["x2_true"] - df_all["x1_true"]
+    df_all["h"] = df_all["y2_true"] - df_all["y1_true"]
+    df_all["diff_rate"] = df_all["diff"] / (df_all["w"] + df_all["h"]) / 2
+    false_positive_idx = df_all[df_all["diff_rate"] > thres].index
     df_all_1 = df_all.copy()
 
     # label mismatch
