@@ -7,6 +7,7 @@
 
 # python packages
 from absl.flags import FLAGS
+from typing import Optional
 
 # 3rd-party packages
 from loguru import logger
@@ -15,7 +16,7 @@ from loguru import logger
 # self-defined packages
 
 @logger.catch(reraise=True)
-def get_flag(attr, default=None):
+def get_flag(attr: str, default: Optional[str] = None) -> str:
     try:
         if hasattr(FLAGS, attr):
             return getattr(FLAGS, attr)
@@ -23,3 +24,7 @@ def get_flag(attr, default=None):
         return default
     return default
 
+
+@logger.catch(reraise=True)
+def log_flag() -> None:
+    logger.debug(f"Flags: {FLAGS.flags_into_string()}")
